/*
 * rpcs - An rpc port scanner.
 *	Usage: rpcs <startingIP> <endingIP>
 *		where endingIP >= startingIP
 *	Note: rpcs loggs ONLY to stdout and stderr, so a command like:
 *		rpcs <ip> <ip> >& log (for (t)csh) or
 *		rpcs <ip> <ip> 2> log (for (ba)sh) will have to be used.
 *	(c) 1996 Swarthy 
 *		Latest revision: 11.10.96
 */

#include	<sys/wait.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<malloc.h>
#include	<stdio.h>
#include	<stdlib.h>

#define		MAX_IP_LEN	15

struct ipoctet {
	char	a[4];
	char	b[4];
	char	c[4];
	char	d[4];
};

struct ipocteti {
	int	a;
	int	b;
	int	c;
	int	d;
};

main(int	ac, char	**av)
{
	int	i, j, a, b, c, d;
	pid_t	pid;
	char	*arg;
	struct ipoctet	*ips, *ipe;
	struct ipocteti ipsi, ipei;

	/* 
	 * Make sure we have proper input 
	 */
	if (av[0] == NULL || av[1] == NULL || 
	 (strlen(av[1]) > MAX_IP_LEN) || (strlen(av[2]) > MAX_IP_LEN) ) {
		fprintf(stderr, "usage: %s <startingIP> <endingIP>\n", av[0]);
		exit(-1);
	}	
	/*
	 * Initialize vars
	 */
	i = j = 0;
	ips = (struct ipoctet *)malloc(sizeof(struct ipoctet));
	ipe = (struct ipoctet *)malloc(sizeof(struct ipoctet));
	

	/* 
	 * Parse each portion of dotted notation IP addr into string 
	 * in ipocted struct for both starting and ending ips.
	 */
	while (av[1][i] != '.') 
		ips->a[j++] = av[1][i++];
	ips->a[j] = '\0';
	j = 0; i++;

	while (av[1][i] != '.') 
		ips->b[j++] = av[1][i++];
	ips->b[j] = '\0';
	j = 0; i++;

	while (av[1][i] != '.') 
		ips->c[j++] = av[1][i++];
	ips->c[j] = '\0';
	j = 0; i++;

	while (av[1][i] != '\0') 
		ips->d[j++] = av[1][i++];
	ips->d[j] = '\0';


/* now for the ending ip.. */
	i = j = 0;

	while (av[2][i] != '.') 
		ipe->a[j++] = av[2][i++];
	ipe->a[j] = '\0';
	j = 0; i++;

	while (av[2][i] != '.') 
		ipe->b[j++] = av[2][i++];
	ipe->b[j] = '\0';
	j = 0; i++;

	while (av[2][i] != '.') 
		ipe->c[j++] = av[2][i++];
	ipe->c[j] = '\0';
	j = 0; i++;

	while (av[2][i] != '\0') 
		ipe->d[j++] = av[2][i++];
	ipe->d[j] = '\0';

	/* 
	 * Convert to integer values and store in struct
	 */
	if (ips->a != NULL) ipsi.a = atoi(ips->a);
	if (ips->b != NULL) ipsi.b = atoi(ips->b);
	if (ips->c != NULL) ipsi.c = atoi(ips->c);
	if (ips->d != NULL) ipsi.d = atoi(ips->d);

	if (ipe->a != NULL) ipei.a = atoi(ipe->a);
	if (ipe->b != NULL) ipei.b = atoi(ipe->b);
	if (ipe->c != NULL) ipei.c = atoi(ipe->c);
	if (ipe->d != NULL) ipei.d = atoi(ipe->d);

	/* free evil ips and ipe structs 
	free(ipe);
	free(ips);
	*/

	/*
	 * Now, report parsings
	 */
	printf("-- Scanning from %d.%d.%d.%d -> %d.%d.%d.%d\n",
		ipsi.a, ipsi.b, ipsi.c, ipsi.d, ipei.a, ipei.b, 
		ipei.c, ipei.d);
	
	for (a = ipsi.a ; a <= ipei.a; a++ ) 
	  for (b = ipsi.b; b <= ipei.b; b++)
	    for (c = ipsi.c; c <= ipei.c; c++) 
	      for (d = ipsi.d; d <= ipei.d; d++) {
		arg = (char *)malloc(17);
		sprintf(arg, "%d.%d.%d.%d", a,b,c,d);
		printf("\t+++rpcinfo -p %s\n",arg);

		pid = fork();
		if (pid == 0) {
			execlp("rpcinfo", "rpcinfo", "-p", arg, NULL);
			exit(0);
		}
		waitpid(pid, NULL, 0);

		free(arg);
		}
	
	exit(0);
}

